/*
 *   This file is part of the Maximum library.
 *
 *   Maximum is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Maximum is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Maximum.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.Text;

namespace Maximum
{
	public class DataStats
	{
		private double m_min = Double.PositiveInfinity;
		private double m_max = Double.NegativeInfinity;
		private double m_mean;
		private int m_n;
		private double m_ssd;

		public DataStats()
		{
		}

		public DataStats(List<double> list)
		{
			Recalc(list);
		}

		public DataStats(double[] list)
		{
			Recalc(list);
		}

		public void Recalc(List<double> list)
		{
			double sum = 0;
            m_min = Double.MaxValue;
            m_max = Double.MinValue;
            m_n = list.Count;
			for (int x = 0; x < m_n; x++)
			{
				double val = list[x];
				if (val < m_min)
				{
					m_min = val;
				}
				if (val > m_max)
				{
					m_max = val;
				}
				sum += val;
			}
			m_mean = sum / m_n;

			sum = 0;
			double mean2 = m_mean * m_mean;
			for (int x = 0; x < m_n; x++)
			{
				double val = list[x];
				sum += (val * val) - mean2;
			}
			m_ssd = Math.Sqrt(sum / m_n);
		}

		public void Recalc(double[] list)
		{
			double sum = 0;
            m_min = Double.MaxValue;
            m_max = Double.MinValue;
			m_n = list.Length;
			for (int x = 0; x < m_n; x++)
			{
				double val = list[x];
				if (val < m_min)
				{
					m_min = val;
				}
				if (val > m_max)
				{
					m_max = val;
				}
				sum += val;
			}
			m_mean = sum / m_n;

			sum = 0;
			double mean2 = m_mean * m_mean;
			for (int x = 0; x < m_n; x++)
			{
				double val = list[x];
				sum += (val * val) - mean2;
			}
			m_ssd = Math.Sqrt(sum / m_n);
		}

		public double Min
		{
			get { return m_min; }
		}

		public double Max
		{
			get { return m_max; }
		}

		public double Mean
		{
			get { return m_mean; }
		}

		public int N
		{
			get { return m_n; }
		}

		public double StDevSample
		{
			get { return m_ssd; }
		}
	}
}
