/*
 *   This file is part of the Maximum library.
 *
 *   Maximum is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Maximum is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Maximum.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;

namespace Maximum.NeuralNet.Backprop
{
	public class DataConfig
	{
		struct DataItem
		{
			public int index;
			public NormalizedData data;
			public string name;
			public int windowLen;
		}

		private List<string> m_names = new List<string>();
		private List<DataItem> m_items = new List<DataItem>();
		private Dictionary<string, DataItem> m_idx = new Dictionary<string, DataItem>();
		private int m_maxWindowLen;
		private int m_inputElements;

		public DataConfig()
		{
		}

		public string[] InputNames()
		{
			string[] names = new string[InputSize];
			int pos = 0;
			foreach (string name in m_names)
			{
				DataItem di = m_idx[name];
				for (int x = 0; x < di.windowLen; x++)
				{
					names[pos++] = String.Format("{0} T-{1}", name, x);
				}
			}
			return names;
		}

		public void Add(NormalizedData data, string name, int windowLen)
		{
			if (windowLen <= 0)
			{
				throw new NeuralNetworkException("Window length must be greater than zero");
			}
			if (windowLen > m_maxWindowLen)
			{
				m_maxWindowLen = windowLen;
			}
			m_inputElements += windowLen;

			DataItem item;
			item.index = m_items.Count;
			item.data = data;
			item.name = name;
			item.windowLen = windowLen;
			m_items.Add(item);
			m_idx.Add(name, item);
			m_names.Add(name);
		}

		public double[] InputAtIndex(int idx)
		{
			double[] data = new double[m_inputElements];
			InputAtIndex(idx, data);
			return data;
		}

		public void InputAtIndex(int idx, double[] input)
		{
			Debug.Assert(input.Length == InputSize);
			if (idx < m_maxWindowLen)
			{
				throw new NeuralNetworkException("The index must be greater than the maximum window size");
			}
			int dpos = 0;
			foreach (DataItem item in m_items)
			{
				for (int x = 0; x < item.windowLen; x++)
				{
					input[dpos++] = item.data[idx - x];
					Debug.Assert(input[dpos - 1] != Double.NaN);
				}
			}
		}

		public NormalizedData this[string idx]
		{
			get { return m_idx[idx].data; }
		}

		public int InputSize
		{
			get { return m_inputElements; }
		}

		public int WindowSize
		{
			get { return m_maxWindowLen; }
		}
	}
}
