/*
 *   This file is part of the Maximum library.
 *
 *   Maximum is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Maximum is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Maximum.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.Text;

namespace Maximum.NeuralNet.Backprop
{
	struct TrainingData
	{
		public double[] input;
		public double[] output;
	}

	public class TrainingSet
	{
		private List<TrainingData> m_data = new List<TrainingData>();

		public TrainingSet()
		{
		}

		public double Train(NeuralNetwork nn, int maxTotalTries, double maxTotalError, int maxTries, double maxError)
		{
			double error = 0;
			for (int i = 0; i < maxTotalTries; i++)
			{
				error = 0;
				for (int x = 0; x < m_data.Count; x++)
				{
					error += nn.Train(m_data[x].input, m_data[x].output, maxError, maxTries);
				}
				error = Test(nn);
				Console.WriteLine("{0}: {1}", i, error);
				if (error <= maxTotalError)
				{
					return error;
				}
			}
			return Test(nn);
		}

		public double Test(NeuralNetwork nn)
		{
			double error = 0;
			for (int x = 0; x < m_data.Count; x++)
			{
				error += nn.CalcError(m_data[x].input, m_data[x].output);
			}
			return error;
		}

		public void AddMapping(double[] input, double[] output)
		{
			TrainingData td;
			td.input = input;
			td.output = output;
			m_data.Add(td);
		}

		public int Count
		{
			get { return m_data.Count; }
		}
	}
}
