﻿USE NNET;

CREATE TABLE NCONN (
	NETWORK_ID INT NOT NULL ,
	LAYER_NUM TINYINT NOT NULL ,
	INDEX_NUM INT NOT NULL ,
	WEIGHT_NUM DOUBLE NOT NULL ,
	DELTA_NUM DOUBLE NOT NULL
);


CREATE TABLE NETWORK (
	NETWORK_ID INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
	LEARN_RATE_NUM DOUBLE NOT NULL ,
	MOMENTUM_NUM DOUBLE NOT NULL ,
	WEIGHT_RANGE_NUM DOUBLE NOT NULL ,
	ADAPT_LEARN_RATE_IND BIT NOT NULL ,
	ADAPT_MOM_IND BIT NOT NULL ,
	PREV_ERROR_NUM DOUBLE NOT NULL ,
	IN_NODE_NUM INT NOT NULL ,
	L1_NODE_NUM INT NOT NULL ,
	L2_NODE_NUM INT NOT NULL ,
	OUT_NODE_NUM INT NOT NULL
);


CREATE TABLE NODE (
	NETWORK_ID INT NOT NULL ,
	LAYER_NUM TINYINT NOT NULL ,
	INDEX_NUM INT NOT NULL ,
	VAL_NUM DOUBLE NOT NULL
);


ALTER TABLE NCONN ADD
	CONSTRAINT PK_NCONN PRIMARY KEY  CLUSTERED
	(
		NETWORK_ID,
		LAYER_NUM,
		INDEX_NUM
	);


ALTER TABLE NODE ADD
	CONSTRAINT PK_NODE PRIMARY KEY  CLUSTERED
	(
		NETWORK_ID,
		LAYER_NUM,
		INDEX_NUM
	);


DELIMITER |
CREATE PROCEDURE debugClear
(
)
BEGIN
DELETE FROM
	NCONN;

DELETE FROM
	NODE;

DELETE FROM
	NETWORK;
END
|
DELIMITER ;


DELIMITER |
CREATE PROCEDURE delNetwork
(
	_NETWORK_ID INT
)
BEGIN

DELETE FROM
	NETWORK
WHERE
	NETWORK_ID = _NETWORK_ID;

DELETE FROM
	NODE
WHERE
	NETWORK_ID = _NETWORK_ID;

DELETE FROM
	NCONN
WHERE
	NETWORK_ID = _NETWORK_ID;
END
|
DELIMITER ;


GRANT  EXECUTE ON PROCEDURE delNetwork  TO 'stocknn'@'%';


DELIMITER |
CREATE PROCEDURE insConn
(
	_NETWORK_ID INT,
	_LAYER_NUM TINYINT, 
	_INDEX_NUM INT, 
	_WEIGHT_NUM DOUBLE,
	_DELTA_NUM DOUBLE
)
BEGIN


INSERT INTO
	NCONN
(
	NETWORK_ID, 
	LAYER_NUM, 
	INDEX_NUM, 
	WEIGHT_NUM, 
	DELTA_NUM
)
VALUES
(
	_NETWORK_ID, 
	_LAYER_NUM, 
	_INDEX_NUM, 
	_WEIGHT_NUM, 
	_DELTA_NUM
);
END
|
DELIMITER ;

GRANT  EXECUTE ON PROCEDURE insConn  TO 'stocknn'@'%';


DELIMITER |
CREATE PROCEDURE insNetwork
(
	OUT _NETWORK_ID INT,
	_LEARN_RATE_NUM DOUBLE,
	_MOMENTUM_NUM DOUBLE,
	_WEIGHT_RANGE_NUM DOUBLE,
	_ADAPT_LEARN_RATE_IND BIT, 
	_ADAPT_MOM_IND BIT, 
	_PREV_ERROR_NUM DOUBLE,
	_IN_NODE_NUM INT, 
	_L1_NODE_NUM INT, 
	_L2_NODE_NUM INT, 
	_OUT_NODE_NUM INT
)
BEGIN

INSERT INTO 
	NETWORK
(
	LEARN_RATE_NUM, 
	MOMENTUM_NUM, 
	WEIGHT_RANGE_NUM, 
	ADAPT_LEARN_RATE_IND, 
	ADAPT_MOM_IND, 
	PREV_ERROR_NUM, 
	IN_NODE_NUM, 
	L1_NODE_NUM, 
	L2_NODE_NUM, 
	OUT_NODE_NUM
)
VALUES
(
	_LEARN_RATE_NUM,
	_MOMENTUM_NUM, 
	_WEIGHT_RANGE_NUM, 
	_ADAPT_LEARN_RATE_IND, 
	_ADAPT_MOM_IND, 
	_PREV_ERROR_NUM, 
	_IN_NODE_NUM, 
	_L1_NODE_NUM, 
	_L2_NODE_NUM, 
	_OUT_NODE_NUM
);

SET _NETWORK_ID = last_insert_id();
END
|
DELIMITER ;

GRANT  EXECUTE ON PROCEDURE insNetwork  TO 'stocknn'@'%';


DELIMITER |
CREATE PROCEDURE insNode
(
	_NETWORK_ID INT, 
	_LAYER_NUM TINYINT, 
	_INDEX_NUM SMALLINT, 
	_VAL_NUM DOUBLE
)
BEGIN

INSERT INTO
	NODE
(
	NETWORK_ID, 
	LAYER_NUM, 
	INDEX_NUM, 
	VAL_NUM
)
VALUES
(
	_NETWORK_ID, 
	_LAYER_NUM,
	_INDEX_NUM, 
	_VAL_NUM
);
END
|
DELIMITER ;

GRANT  EXECUTE ON PROCEDURE insNode  TO 'stocknn'@'%';


DELIMITER |
CREATE PROCEDURE selConns
(
	_NETWORK_ID INT
)
BEGIN

SELECT
	*
FROM
	NCONN
WHERE
	NETWORK_ID = _NETWORK_ID;
END
|
DELIMITER ;

GRANT  EXECUTE ON PROCEDURE selConns  TO 'stocknn'@'%';

DELIMITER |
CREATE PROCEDURE selNetwork
(
	_NETWORK_ID INT
)
BEGIN

SELECT
	*
FROM
	NETWORK
WHERE
	NETWORK_ID = _NETWORK_ID;
END
|
DELIMITER ;

GRANT  EXECUTE ON PROCEDURE selNetwork  TO 'stocknn'@'%';


DELIMITER |
CREATE PROCEDURE selNodes
(
	_NETWORK_ID INT
)
BEGIN

SELECT
	*
FROM
	NODE
WHERE
	NETWORK_ID = _NETWORK_ID;
END
|
DELIMITER ;

GRANT  EXECUTE ON PROCEDURE selNodes  TO 'stocknn'@'%';


DELIMITER |
CREATE PROCEDURE updConn
(
	_NETWORK_ID INT, 
	_LAYER_NUM TINYINT, 
	_INDEX_NUM INT, 
	_WEIGHT_NUM DOUBLE,
	_DELTA_NUM DOUBLE
)
BEGIN

UPDATE
	NCONN
SET
	WEIGHT_NUM = _WEIGHT_NUM, 
	DELTA_NUM = DELTA_NUM
WHERE
	NETWORK_ID = _NETWORK_ID AND
	LAYER_NUM = _LAYER_NUM AND
	INDEX_NUM = _INDEX_NUM
;
SELECT FOUND_ROWS();
END
|
DELIMITER ;

GRANT  EXECUTE ON PROCEDURE updConn  TO 'stocknn'@'%';


DELIMITER |
CREATE PROCEDURE updNode
(
	_NETWORK_ID INT, 
	_LAYER_NUM TINYINT, 
	_INDEX_NUM SMALLINT, 
	_VAL_NUM DOUBLE
)
BEGIN
UPDATE
	NODE
SET
	VAL_NUM = _VAL_NUM
WHERE
	NETWORK_ID = _NETWORK_ID AND 
	LAYER_NUM = _LAYER_NUM AND
	INDEX_NUM = _INDEX_NUM
;
SELECT ROW_COUNT();
END
|
DELIMITER ;

GRANT  EXECUTE ON PROCEDURE updNode  TO 'stocknn'@'%';

GRANT SELECT ON TABLE mysql.proc TO 'stocknn'@'%';
