/*
 *   This file is part of the Maximum library.
 *
 *   Maximum is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Maximum is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Maximum.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace Maximum
{
	/// <summary>
	/// In garbage collected environments, pooling frequently allocated objects
	/// can greatly improve performance.
	/// </summary>
	public class ObjectPool<T> where T : new()
	{
		private Stack<T> m_pool = new Stack<T>();

		public ObjectPool()
		{
			for (int x = 0; x < 5; x++)
			{
				m_pool.Push(new T());
			}
		}

		public T Get()
		{
			lock (m_pool)
			{
				if (m_pool.Count == 0)
				{
					return new T();
				}
				return m_pool.Pop();
			}
		}

		public void Release(T t)
		{
			lock (m_pool)
			{
				m_pool.Push(t);
			}
		}
	}
}
