// StringTable.cs created with MonoDevelop
// User: john at 3:17 P 22/04/2008
//
//   This file is part of the Maximum Library.
//
//   Maximum is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.
//
//   Maximum is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with Maximum.  If not, see <http://www.gnu.org/licenses/>.
//

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;

namespace Maximum
{
	/// <summary>
	/// A string table can be used to eliminate redundant string allocations.
	/// </summary>
	public class StringTable
	{
		private Dictionary<int, List<string>> m_strings = new Dictionary<int, List<string>>();

		public StringTable()
		{
		}
		
		public string Get(StringBuilder sb)
		{
			int hash = Hash(sb);
			if ( !m_strings.ContainsKey(hash) )
			{
				m_strings.Add(hash, new List<string>());
			}
			List<string> lst = m_strings[hash];
			string str = null;
			if ( ListContains(lst, sb, ref str) )
			{
				return str;
			}
			str = sb.ToString();
			lst.Add(str);
			return str;
		}
		
		private bool ListContains(List<string> lst, StringBuilder sb, ref string str)
		{
			for ( int x = 0; x < lst.Count; x++ )
			{
				if ( StringHelper.AreEqual(sb, lst[x]) )
				{
					str = lst[x];
					return true;
				}
			}
			return false;
		}

		public string Get(string str)
		{
			int hash = Hash(str);
			if ( !m_strings.ContainsKey(hash) )
			{
				m_strings.Add(hash, new List<string>());
			}
			List<string> lst = m_strings[hash];
			if ( ListContains(lst, ref str) )
			{
				return str;
			}
			lst.Add(str);
			return str;
		}
		
		private bool ListContains(List<string> lst, ref string str)
		{
			for ( int x = 0; x < lst.Count; x++ )
			{
				if ( lst[x].Equals(str) )
				{
					str = lst[x];
					return true;
				}
			}
			return false;
		}
		
		private static int Hash(string str)
		{
			int hash = 0;
			int len = str.Length;
			int lenmod4 = len % 4;
			int part;
		
			Debug.Assert( (len - lenmod4)%4 == 0 );
			for ( int x = 0; x < len - lenmod4; x+=4 )
			{
				part = str[x];
		
				part |= (int)str[x+1] << 8;
		
				part |= (int)str[x+2] << 16;
		
				part |= (int)str[x+3] << 24;
		
				hash ^= part;
			}
			part = 0;
			if ( lenmod4 > 2 )
			{
				part |= (int)str[len-3];
			}
			if ( lenmod4 > 1 )
			{
				part |= (int)str[len-2];
			}
			if ( lenmod4 > 0 )
			{
				part |= (int)str[len-1];
				hash ^= part;
			}
			if ( hash < 0 )
			{
				return -hash;
			}
			return hash;
		}
		
		private static int Hash(StringBuilder str)
		{
			int hash = 0;
			int len = str.Length;
			int lenmod4 = len % 4;
			int part;
		
			Debug.Assert( (len - lenmod4)%4 == 0 );
			for ( int x = 0; x < len - lenmod4; x+=4 )
			{
				part = str[x];
		
				part |= (int)str[x+1] << 8;
		
				part |= (int)str[x+2] << 16;
		
				part |= (int)str[x+3] << 24;
		
				hash ^= part;
			}
			part = 0;
			if ( lenmod4 > 2 )
			{
				part |= (int)str[len-3];
			}
			if ( lenmod4 > 1 )
			{
				part |= (int)str[len-2];
			}
			if ( lenmod4 > 0 )
			{
				part |= (int)str[len-1];
				hash ^= part;
			}
			if ( hash < 0 )
			{
				return -hash;
			}
			return hash;		
		}
	}
}
