// DelimlLatTest.cs created with MonoDevelop
// User: john at 6:46 P 29/03/2008
/*
 *   This file is part of the Maximum Library.
 *
 *   Maximum is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Maximum is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Maximum.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.IO;

using NUnit.Framework;

namespace Maximum
{
	[TestFixture]
	public class DelimlLatTest
	{		
		public DelimlLatTest()
		{
		}
		
		[Test]
		public void Case1()
		{
			MemoryStream min = new MemoryStream();
			MemoryStream mout = new MemoryStream();
			
			TextWriter tw = new StreamWriter(min);
			tw.WriteLine("a,b,c");
			tw.WriteLine("dd,ee,ff");
			
			tw = new StreamWriter(mout);
			TextReader tr = new StreamReader(min);
			
			DelimitedFileLat csv = new DelimitedFileLat(',', 3, tr, tw);
			
			Assert.IsTrue(csv.Next(), "Should have row one");
			Assert.AreEqual(csv.Column(0), "a");
			Assert.AreEqual(csv.Column(1), "b");
			Assert.AreEqual(csv.Column(2), "c");
			
			Assert.IsTrue(csv.Next(), "Should have row two");
			Assert.AreEqual(csv.Column(0), "dd");
			Assert.AreEqual(csv.Column(1), "ee");
			Assert.AreEqual(csv.Column(2), "ff");

			Assert.IsFalse(csv.Next(), "EOF");
			
			Assert.AreEqual(mout.ReadByte(), (byte)'a');
			Assert.AreEqual(mout.ReadByte(), (byte)'c');
			Assert.AreEqual(mout.ReadByte(), (byte)'d');
			Assert.AreEqual(mout.ReadByte(), (byte)'\n');
			Assert.AreEqual(mout.ReadByte(), (byte)'d');
			Assert.AreEqual(mout.ReadByte(), (byte)'d');
			Assert.AreEqual(mout.ReadByte(), (byte)'e');
			Assert.AreEqual(mout.ReadByte(), (byte)'e');
			Assert.AreEqual(mout.ReadByte(), (byte)'f');
			Assert.AreEqual(mout.ReadByte(), (byte)'f');
			Assert.AreEqual(mout.ReadByte(), (byte)'\n');
		}

		[Test]
		public void Quotes()
		{
			MemoryStream min = new MemoryStream();
			MemoryStream mout = new MemoryStream();
			
			TextWriter tw = new StreamWriter(min);
			tw.WriteLine("a,\"b\",\"c\"");
			tw.WriteLine("\"d\",ee,ff");
			
			tw = new StreamWriter(mout);
			TextReader tr = new StreamReader(min);
			
			DelimitedFileLat csv = new DelimitedFileLat(',', 3, tr, tw);
			
			Assert.IsTrue(csv.Next(), "Should have row one");
			Assert.AreEqual(csv.Column(0), "a");
			Assert.AreEqual(csv.Column(1), "b");
			Assert.AreEqual(csv.Column(2), "c");
			
			Assert.IsTrue(csv.Next(), "Should have row two");
			Assert.AreEqual(csv.Column(0), "dd");
			Assert.AreEqual(csv.Column(1), "ee");
			Assert.AreEqual(csv.Column(2), "ff");

			Assert.IsFalse(csv.Next(), "EOF");
			
			Assert.AreEqual(mout.ReadByte(), (byte)'a');
			Assert.AreEqual(mout.ReadByte(), (byte)'c');
			Assert.AreEqual(mout.ReadByte(), (byte)'d');
			Assert.AreEqual(mout.ReadByte(), (byte)'\n');
			Assert.AreEqual(mout.ReadByte(), (byte)'d');
			Assert.AreEqual(mout.ReadByte(), (byte)'d');
			Assert.AreEqual(mout.ReadByte(), (byte)'e');
			Assert.AreEqual(mout.ReadByte(), (byte)'e');
			Assert.AreEqual(mout.ReadByte(), (byte)'f');
			Assert.AreEqual(mout.ReadByte(), (byte)'f');
			Assert.AreEqual(mout.ReadByte(), (byte)'\n');
		}
	}
}
