using System;
using System.Configuration;
using System.Data;
//using System.Data.SqlClient;
using NUnit.Framework;

using MySql.Data;
using MySql.Data.MySqlClient;

using Maximum.NeuralNet.Backprop;

namespace NnBackprop.UnitTest
{
	[TestFixture]
	public class XorTest
	{
		double errorTarget = .0001;

		double[] in1 = {1.0, 1.0};
		double[] out1 = {0.0};

		double[] in2 = {1.0, 0.0};
		double[] out2 = {1.0};

		double[] in3 = {0.0, 1.0};
		double[] out3 = {1.0};

		double[] in4 = {0.0, 0.0};
		double[] out4 = {0.0};

		public XorTest()
		{
		}

		private double TrainXor(NeuralNetwork nn, double errorTarget)
		{
			double error = 0;

			for ( int x = 0; x < 20000; x++ )
			{
				error = nn.Train(in2, out2, errorTarget, 20000);
				error += nn.Train(in3, out3, errorTarget, 20000);
				error += nn.Train(in1, out1, errorTarget, 20000);
				error += nn.Train(in4, out4, errorTarget, 20000);
				if ( error <= errorTarget )
				{
					break;
				}
			}
			return error;
		}

		[Test]
		public void XOR()
		{
			NeuralNetwork nn = new NeuralNetwork( 2, 4, 4, 1, true, false );
			double error = TrainXor( nn, errorTarget );
			Assert.IsTrue( error <= errorTarget, "Network didn't learn; error=" + error );
		}

		[Test]
		public void LoadSave()
		{
			NeuralNetwork nn = new NeuralNetwork( 2, 4, 4, 1, true, false );
			double errorTarget = .0001;
			double error = TrainXor( nn, errorTarget );
			Assert.IsTrue( error <= errorTarget, "Network didn't learn; error=" + error );

			MySqlConnection con = new MySqlConnection( "server=athlon;user id=stocknn;pwd=xyzzy;database=stock" );	
			con.Open();
			nn.Write( con );
			int networkId = nn.NetworkId;
			nn = new NeuralNetwork( networkId, con );

			error = nn.CalcError(in2, out2);
			error += nn.CalcError(in3, out3);
			error += nn.CalcError(in1, out1);
			error += nn.CalcError(in4, out4);
			
			Assert.IsTrue( error <= errorTarget, "Network didn't save correctly; error=" + error );

			MySqlCommand cmd = new MySqlCommand( "delNetwork", con );
			cmd.CommandType = CommandType.StoredProcedure;
			cmd.Parameters.AddWithValue("_NETWORK_ID", networkId);
			cmd.ExecuteNonQuery();
			con.Close();
		}
	}
}
