using System;
using System.Collections.Generic;
using System.Text;

using NUnit.Framework;

namespace Maximum.UnitTest
{
	[TestFixture]
	public class ProgArgTest
	{
		[Test]
		public void TestSlashStyle()
		{
			ProgramArguments args = new ProgramArguments(new string[] { "/t:100", "/a:mom", "/?", "filename" });
			Assert.AreEqual(args.Count, 3);
			Assert.IsTrue(args.HasSwitch("t"));
			Assert.IsTrue(args.HasSwitch("a"));
			Assert.AreEqual(args["t"], "100");
			Assert.AreEqual(args["a"], "mom");
			Assert.IsTrue(args.HasSwitch("?"));
			Assert.AreEqual(args.Parameters[0], "filename");
		}

		[Test]
		public void TestDashDashStyle()
		{
			ProgramArguments args = new ProgramArguments(new string[] { "--t=100", "--a=mom", "--help", "filename" });
			Assert.AreEqual(args.Count, 3);
			Assert.IsTrue(args.HasSwitch("t"));
			Assert.IsTrue(args.HasSwitch("a"));
			Assert.AreEqual(args["t"], "100");
			Assert.AreEqual(args["a"], "mom");
			Assert.IsTrue(args.HasSwitch("help"));
			Assert.AreEqual(args.Parameters[0], "filename");
		}

		[Test]
		public void TestDashStyle()
		{
			ProgramArguments args = new ProgramArguments(new string[] { "-t", "100", "-a", "mom", "-?", "filename" });
			Assert.AreEqual(args.Count, 3);
			Assert.IsTrue(args.HasSwitch("t"));
			Assert.IsTrue(args.HasSwitch("a"));
			Assert.IsTrue(args["t"] == "");
			Assert.IsTrue(args["a"] == "");
			Assert.IsTrue(args.HasSwitch("?"));
			Assert.AreEqual(args.Parameters.Length, 3);
			Assert.AreEqual(args.Parameters[0], "100");
			Assert.AreEqual(args.Parameters[1], "mom");
			Assert.AreEqual(args.Parameters[2], "filename");
		}

		[Test]
		public void TestQuotes()
		{
			ProgramArguments args = new ProgramArguments(new string[] { "\"--t=now is the time\"" });
			Assert.AreEqual(args.Count, 1);
			Assert.IsTrue(args.HasSwitch("t"));
			Assert.AreEqual(args["t"], "now is the time");

			args = new ProgramArguments(new string[] { "--t=\"now is the time\"" });
			Assert.AreEqual(args.Count, 1);
			Assert.IsTrue(args.HasSwitch("t"));
			Assert.AreEqual(args["t"], "now is the time");

			args = new ProgramArguments(new string[] { "-t", "\"now is the time\"" });
			Assert.AreEqual(args.Count, 1);
			Assert.IsTrue(args.HasSwitch("t"));
			Assert.AreEqual(args.Parameters[0], "now is the time");
		}

		[Test]
		public void TestVerify()
		{
			try
			{
				new ProgramArguments(new string[] { "--t=100", "--a=mom", "filename" }, "t,a");
			}
			catch (ProgramArgumentException)
			{
				Assert.Fail();
			}
			try
			{
				new ProgramArguments(new string[] { "--t=100", "--a=mom", "filename" }, "a");
				Assert.Fail();
			}
			catch (ProgramArgumentException)
			{
			}
			try
			{
				new ProgramArguments(new string[] { "--t=100", "--a=mom", "filename" }, "t");
				Assert.Fail();
			}
			catch (ProgramArgumentException)
			{
			}
		}
	}
}
